@@ -24,9 +24,6 @@ struct Base_extra_yes
   {
   arma_inline const Op<derived,op_inv> i() const;   //!< matrix inverse
   
-  arma_deprecated inline const Op<derived,op_inv> i(const bool ) const;   //!< kept only for compatibility with old user code
-  arma_deprecated inline const Op<derived,op_inv> i(const char*) const;   //!< kept only for compatibility with old user code
-  
   inline arma_warn_unused bool is_sympd() const;
   inline arma_warn_unused bool is_sympd(typename get_pod_type<elem_type>::result tol) const;
   };
@@ -140,14 +137,19 @@ struct Base
   inline arma_warn_unused bool is_hermitian() const;
   inline arma_warn_unused bool is_hermitian(const typename get_pod_type<elem_type>::result tol) const;
   
-  inline arma_warn_unused bool is_empty()  const;
-  inline arma_warn_unused bool is_square() const;
-  inline arma_warn_unused bool is_vec()    const;
-  inline arma_warn_unused bool is_colvec() const;
-  inline arma_warn_unused bool is_rowvec() const;
-  inline arma_warn_unused bool is_finite() const;
-  inline arma_warn_unused bool has_inf()   const;
-  inline arma_warn_unused bool has_nan()   const;
+  inline arma_warn_unused bool is_zero(const typename get_pod_type<elem_type>::result tol = 0) const;
+  
+  inline arma_warn_unused bool is_trimatu() const;
+  inline arma_warn_unused bool is_trimatl() const;
+  inline arma_warn_unused bool is_diagmat() const;
+  inline arma_warn_unused bool is_empty()   const;
+  inline arma_warn_unused bool is_square()  const;
+  inline arma_warn_unused bool is_vec()     const;
+  inline arma_warn_unused bool is_colvec()  const;
+  inline arma_warn_unused bool is_rowvec()  const;
+  inline arma_warn_unused bool is_finite()  const;
+  inline arma_warn_unused bool has_inf()    const;
+  inline arma_warn_unused bool has_nan()    const;
   
   arma_inline const Op<derived,op_vectorise_col> as_col() const;
   arma_inline const Op<derived,op_vectorise_row> as_row() const;
