@@ -19,16 +19,11 @@
 
 
 #if defined(ARMA_RNG_ALT)
-  #undef ARMA_USE_EXTERN_CXX11_RNG
+  #undef ARMA_USE_EXTERN_RNG
 #endif
 
 
-#if !defined(ARMA_USE_CXX11)
-  #undef ARMA_USE_EXTERN_CXX11_RNG
-#endif
-
-
-#if defined(ARMA_USE_EXTERN_CXX11_RNG)
+#if defined(ARMA_USE_EXTERN_RNG)
   extern thread_local arma_rng_cxx11 arma_rng_cxx11_instance;
   // namespace { thread_local arma_rng_cxx11 arma_rng_cxx11_instance; }
 #endif
@@ -40,18 +35,18 @@ class arma_rng
   
   #if   defined(ARMA_RNG_ALT)
     typedef arma_rng_alt::seed_type   seed_type;
-  #elif defined(ARMA_USE_EXTERN_CXX11_RNG)
+  #elif defined(ARMA_USE_EXTERN_RNG)
     typedef arma_rng_cxx11::seed_type seed_type;
   #else
     typedef arma_rng_cxx98::seed_type seed_type;
   #endif
   
   #if   defined(ARMA_RNG_ALT)
-    static const int rng_method = 2;
-  #elif defined(ARMA_USE_EXTERN_CXX11_RNG)
-    static const int rng_method = 1;
+    static constexpr int rng_method = 2;
+  #elif defined(ARMA_USE_EXTERN_RNG)
+    static constexpr int rng_method = 1;
   #else
-    static const int rng_method = 0;
+    static constexpr int rng_method = 0;
   #endif
   
   inline static void set_seed(const seed_type val);
@@ -72,7 +67,7 @@ arma_rng::set_seed(const arma_rng::seed_type val)
     {
     arma_rng_alt::set_seed(val);
     }
-  #elif defined(ARMA_USE_EXTERN_CXX11_RNG)
+  #elif defined(ARMA_USE_EXTERN_RNG)
     {
     arma_rng_cxx11_instance.set_seed(val);
     }
@@ -94,23 +89,18 @@ arma_rng::set_seed_random()
   seed_type seed2 = seed_type(0);
   seed_type seed3 = seed_type(0);
   seed_type seed4 = seed_type(0);
-  seed_type seed5 = seed_type(0);
   
   bool have_seed = false;
   
-  #if defined(ARMA_USE_CXX11)
+  try
     {
-    try
-      {
-      std::random_device rd;
-      
-      if(rd.entropy() > double(0))  { seed1 = static_cast<seed_type>( rd() ); }
-      
-      if(seed1 != seed_type(0))  { have_seed = true; }
-      }
-    catch(...) {}
+    std::random_device rd;
+    
+    if(rd.entropy() > double(0))  { seed1 = static_cast<seed_type>( rd() ); }
+    
+    if(seed1 != seed_type(0))  { have_seed = true; }
     }
-  #endif
+  catch(...) {}
   
   
   if(have_seed == false)
@@ -144,17 +134,11 @@ arma_rng::set_seed_random()
     {
     // get better-than-nothing seeds in case reading /dev/urandom failed 
     
-    #if defined(ARMA_HAVE_GETTIMEOFDAY)
-      {
-      struct timeval posix_time;
-      
-      gettimeofday(&posix_time, 0);
-      
-      seed3 = static_cast<seed_type>(posix_time.tv_usec);
-      }
-    #endif
+    const std::chrono::system_clock::time_point tp_now = std::chrono::system_clock::now();
     
-    seed4 = static_cast<seed_type>( std::time(NULL) & 0xFFFF );
+    auto since_epoch_usec = std::chrono::duration_cast<std::chrono::microseconds>(tp_now.time_since_epoch()).count();
+    
+    seed3 = static_cast<seed_type>( since_epoch_usec & 0xFFFF );
     
     union
       {
@@ -164,15 +148,15 @@ arma_rng::set_seed_random()
     
     tmp.a = (uword*)malloc(sizeof(uword));
     
-    if(tmp.a != NULL)
+    if(tmp.a != nullptr)
       {
-      for(size_t i=0; i<sizeof(uword*); ++i)  { seed5 += seed_type(tmp.b[i]); }
+      for(size_t i=0; i<sizeof(uword*); ++i)  { seed4 += seed_type(tmp.b[i]); }
       
       free(tmp.a);
       }
     }
   
-  arma_rng::set_seed( seed1 + seed2 + seed3 + seed4 + seed5 );
+  arma_rng::set_seed( seed1 + seed2 + seed3 + seed4 );
   }
 
 
@@ -187,7 +171,7 @@ struct arma_rng::randi
       {
       return eT( arma_rng_alt::randi_val() );
       }
-    #elif defined(ARMA_USE_EXTERN_CXX11_RNG)
+    #elif defined(ARMA_USE_EXTERN_RNG)
       {
       return eT( arma_rng_cxx11_instance.randi_val() );
       }
@@ -208,7 +192,7 @@ struct arma_rng::randi
       {
       return arma_rng_alt::randi_max_val();
       }
-    #elif defined(ARMA_USE_EXTERN_CXX11_RNG)
+    #elif defined(ARMA_USE_EXTERN_RNG)
       {
       return arma_rng_cxx11::randi_max_val();
       }
@@ -227,15 +211,15 @@ struct arma_rng::randi
     {
     #if   defined(ARMA_RNG_ALT)
       {
-      return arma_rng_alt::randi_fill(mem, N, a, b);
+      arma_rng_alt::randi_fill(mem, N, a, b);
       }
-    #elif defined(ARMA_USE_EXTERN_CXX11_RNG)
+    #elif defined(ARMA_USE_EXTERN_RNG)
       {
-      return arma_rng_cxx11_instance.randi_fill(mem, N, a, b);
+      arma_rng_cxx11_instance.randi_fill(mem, N, a, b);
       }
     #else
       {
-      return arma_rng_cxx98::randi_fill(mem, N, a, b);
+      arma_rng_cxx98::randi_fill(mem, N, a, b);
       }
     #endif
     }
@@ -253,7 +237,7 @@ struct arma_rng::randu
       {
       return eT( arma_rng_alt::randu_val() );
       }
-    #elif defined(ARMA_USE_EXTERN_CXX11_RNG)
+    #elif defined(ARMA_USE_EXTERN_RNG)
       {
       return eT( arma_rng_cxx11_instance.randu_val() );
       }
@@ -270,20 +254,20 @@ struct arma_rng::randu
   void
   fill(eT* mem, const uword N)
     {
-    uword i,j;
+    uword j;
     
-    for(i=0, j=1; j < N; i+=2, j+=2)
+    for(j=1; j < N; j+=2)
       {
       const eT tmp_i = eT( arma_rng::randu<eT>() );
       const eT tmp_j = eT( arma_rng::randu<eT>() );
       
-      mem[i] = tmp_i;
-      mem[j] = tmp_j;
+      (*mem) = tmp_i;  mem++;
+      (*mem) = tmp_j;  mem++;
       }
     
-    if(i < N)
+    if((j-1) < N)
       {
-      mem[i] = eT( arma_rng::randu<eT>() );
+      (*mem) = eT( arma_rng::randu<eT>() );
       }
     }
   };
@@ -330,7 +314,7 @@ struct arma_rng::randn
       {
       return eT( arma_rng_alt::randn_val() );
       }
-    #elif defined(ARMA_USE_EXTERN_CXX11_RNG)
+    #elif defined(ARMA_USE_EXTERN_RNG)
       {
       return eT( arma_rng_cxx11_instance.randn_val() );
       }
@@ -351,7 +335,7 @@ struct arma_rng::randn
       {
       arma_rng_alt::randn_dual_val(out1, out2);
       }
-    #elif defined(ARMA_USE_EXTERN_CXX11_RNG)
+    #elif defined(ARMA_USE_EXTERN_RNG)
       {
       arma_rng_cxx11_instance.randn_dual_val(out1, out2);
       }
@@ -366,7 +350,7 @@ struct arma_rng::randn
   inline
   static
   void
-  fill(eT* mem, const uword N)
+  fill_simple(eT* mem, const uword N)
     {
     uword i, j;
     
@@ -381,6 +365,56 @@ struct arma_rng::randn
       }
     }
   
+  
+  inline
+  static
+  void
+  fill(eT* mem, const uword N)
+    {
+    #if defined(ARMA_USE_OPENMP)
+      {
+      if((N < 1024) || omp_in_parallel())  { arma_rng::randn<eT>::fill_simple(mem, N); return; }
+      
+      typedef std::mt19937_64::result_type seed_type;
+      
+      const uword n_threads = uword( mp_thread_limit::get() );
+      
+      std::vector< std::mt19937_64                  > engine(n_threads);
+      std::vector< std::normal_distribution<double> >  distr(n_threads);
+      
+      for(uword t=0; t < n_threads; ++t)
+        {
+        std::mt19937_64& t_engine = engine[t];
+        
+        t_engine.seed( seed_type(t) + seed_type(arma_rng::randi<seed_type>()) );
+        }
+      
+      const uword chunk_size = N / n_threads;
+      
+      #pragma omp parallel for schedule(static) num_threads(int(n_threads))
+      for(uword t=0; t < n_threads; ++t)
+        {
+        const uword start = (t+0) * chunk_size;
+        const uword endp1 = (t+1) * chunk_size;
+        
+        std::mt19937_64&                  t_engine = engine[t];
+        std::normal_distribution<double>& t_distr  =  distr[t];
+        
+        for(uword i=start; i < endp1; ++i)  { mem[i] = eT( t_distr(t_engine)); }
+        }
+      
+      std::mt19937_64&                  t0_engine = engine[0];
+      std::normal_distribution<double>& t0_distr  =  distr[0];
+      
+      for(uword i=(n_threads*chunk_size); i < N; ++i)  { mem[i] = eT( t0_distr(t0_engine)); }
+      }
+    #else
+      {
+      arma_rng::randn<eT>::fill_simple(mem, N);
+      }
+    #endif
+    }
+  
   };
 
 
@@ -391,7 +425,15 @@ struct arma_rng::randn< std::complex<T> >
   inline
   operator std::complex<T> () const
     {
-    T a, b;
+    #if defined(_MSC_VER)
+      // attempt at workaround for MSVC bug
+      // does MS even test their so-called compilers before release?
+      T a;
+      T b;
+    #else
+      T a(0);
+      T b(0);
+    #endif
     
     arma_rng::randn<T>::dual_val(a, b);
     
@@ -402,7 +444,7 @@ struct arma_rng::randn< std::complex<T> >
   inline
   static
   void
-  fill(std::complex<T>* mem, const uword N)
+  fill_simple(std::complex<T>* mem, const uword N)
     {
     for(uword i=0; i < N; ++i)
       {
@@ -410,6 +452,67 @@ struct arma_rng::randn< std::complex<T> >
       }
     }
   
+  
+  inline
+  static
+  void
+  fill(std::complex<T>* mem, const uword N)
+    {
+    #if defined(ARMA_USE_OPENMP)
+      {
+      if((N < 512) || omp_in_parallel())  { arma_rng::randn< std::complex<T> >::fill_simple(mem, N); return; }
+      
+      typedef std::mt19937_64::result_type seed_type;
+      
+      const uword n_threads = uword( mp_thread_limit::get() );
+      
+      std::vector< std::mt19937_64                  > engine(n_threads);
+      std::vector< std::normal_distribution<double> >  distr(n_threads);
+      
+      for(uword t=0; t < n_threads; ++t)
+        {
+        std::mt19937_64& t_engine = engine[t];
+        
+        t_engine.seed( seed_type(t) + seed_type(arma_rng::randi<seed_type>()) );
+        }
+      
+      const uword chunk_size = N / n_threads;
+      
+      #pragma omp parallel for schedule(static) num_threads(int(n_threads))
+      for(uword t=0; t < n_threads; ++t)
+        {
+        const uword start = (t+0) * chunk_size;
+        const uword endp1 = (t+1) * chunk_size;
+        
+        std::mt19937_64&                  t_engine = engine[t];
+        std::normal_distribution<double>& t_distr  =  distr[t];
+        
+        for(uword i=start; i < endp1; ++i)
+          {
+          const T val1 = T( t_distr(t_engine) );
+          const T val2 = T( t_distr(t_engine) );
+          
+          mem[i] = std::complex<T>(val1, val2);
+          }
+        }
+      
+      std::mt19937_64&                  t0_engine = engine[0];
+      std::normal_distribution<double>& t0_distr  =  distr[0];
+      
+      for(uword i=(n_threads*chunk_size); i < N; ++i)
+        {
+        const T val1 = T( t0_distr(t0_engine) );
+        const T val2 = T( t0_distr(t0_engine) );
+        
+        mem[i] = std::complex<T>(val1, val2);
+        }
+      }
+    #else
+      {
+      arma_rng::randn< std::complex<T> >::fill_simple(mem, N);
+      }
+    #endif
+    }
   };
 
 
