@@ -28,7 +28,7 @@ lu
          Mat<typename T1::elem_type>&    L,
          Mat<typename T1::elem_type>&    U,
   const Base<typename T1::elem_type,T1>& X,
-  const typename arma_blas_type_only<typename T1::elem_type>::result* junk = 0
+  const typename arma_blas_type_only<typename T1::elem_type>::result* junk = nullptr
   )
   {
   arma_extra_debug_sigprint();
@@ -40,8 +40,8 @@ lu
   
   if(status == false)
     {
-    L.reset();
-    U.reset();
+    L.soft_reset();
+    U.soft_reset();
     arma_debug_warn("lu(): decomposition failed");
     }
   
@@ -60,7 +60,7 @@ lu
          Mat<typename T1::elem_type>&    U, 
          Mat<typename T1::elem_type>&    P,
   const Base<typename T1::elem_type,T1>& X,
-  const typename arma_blas_type_only<typename T1::elem_type>::result* junk = 0
+  const typename arma_blas_type_only<typename T1::elem_type>::result* junk = nullptr
   )
   {
   arma_extra_debug_sigprint();
@@ -72,9 +72,9 @@ lu
   
   if(status == false)
     {
-    L.reset();
-    U.reset();
-    P.reset();
+    L.soft_reset();
+    U.soft_reset();
+    P.soft_reset();
     arma_debug_warn("lu(): decomposition failed");
     }
   
