@@ -23,7 +23,15 @@
 class glue_kron
   {
   public:
-
+  
+  template<typename T1, typename T2>
+  struct traits
+    {
+    static constexpr bool is_row  = (T1::is_row && T2::is_row);
+    static constexpr bool is_col  = (T1::is_col && T2::is_col);
+    static constexpr bool is_xvec = false;
+    };
+  
   template<typename eT> inline static void direct_kron(Mat<eT>&                out, const Mat<eT>&                A, const Mat<eT>&                B);
   template<typename T>  inline static void direct_kron(Mat< std::complex<T> >& out, const Mat< std::complex<T> >& A, const Mat<T>&                 B);
   template<typename T>  inline static void direct_kron(Mat< std::complex<T> >& out, const Mat<T>&                 A, const Mat< std::complex<T> >& B);
@@ -34,4 +42,3 @@ class glue_kron
 
 
 //! @}
-
