@@ -36,7 +36,7 @@ op_logmat::apply(Mat< std::complex<typename T1::elem_type> >& out, const mtOp<st
   
   if(status == false)
     {
-    out.reset();
+    out.soft_reset();
     arma_stop_runtime_error("logmat(): transformation failed");
     }
   }
@@ -58,7 +58,7 @@ op_logmat::apply_direct(Mat< std::complex<typename T1::elem_type> >& out, const
   
   const uword N = P.n_rows;
   
-  out.zeros(N,N);
+  out.zeros(N,N);  // aliasing can't happen as op_logmat is defined as cx_mat = op(mat)
   
   for(uword i=0; i<N; ++i)
     {
@@ -89,35 +89,99 @@ op_logmat::apply_direct(Mat< std::complex<typename T1::elem_type> >& out, const
   typedef typename T1::elem_type       in_T;
   typedef typename std::complex<in_T> out_T;
   
-  const Proxy<T1> P(expr.get_ref());
+  const quasi_unwrap<T1> expr_unwrap(expr.get_ref());
+  const Mat<in_T>& A   = expr_unwrap.M;
   
-  arma_debug_check( (P.get_n_rows() != P.get_n_cols()), "logmat(): given matrix must be square sized" );
+  arma_debug_check( (A.is_square() == false), "logmat(): given matrix must be square sized" );
   
-  if(P.get_n_elem() == 0)
+  if(A.n_elem == 0)
     {
     out.reset();
     return true;
     }
   else
-  if(P.get_n_elem() == 1)
+  if(A.n_elem == 1)
     {
     out.set_size(1,1);
-    out[0] = std::log( std::complex<in_T>( P[0] ) );
+    out[0] = std::log( std::complex<in_T>( A[0] ) );
     return true;
     }
   
-  typename Proxy<T1>::ea_type Pea = P.get_ea();
+  if(A.is_diagmat())
+    {
+    const uword N = A.n_rows;
+    
+    out.zeros(N,N);  // aliasing can't happen as op_logmat is defined as cx_mat = op(mat)
+    
+    for(uword i=0; i<N; ++i)
+      {
+      const in_T val = A.at(i,i);
+      
+      if(val >= in_T(0))
+        {
+        out.at(i,i) = std::log(val);
+        }
+      else
+        {
+        out.at(i,i) = std::log( out_T(val) );
+        }
+      }
+    
+    return true;
+    }
   
-  Mat<out_T> U;
-  Mat<out_T> S(P.get_n_rows(), P.get_n_rows());
+  #if defined(ARMA_OPTIMISE_SYMPD)
+    const bool try_sympd = sympd_helper::guess_sympd_anysize(A);
+  #else
+    const bool try_sympd = false;
+  #endif
   
-  out_T* Smem = S.memptr();
+  if(try_sympd)
+    {
+    // if matrix A is sympd, all its eigenvalues are positive
+    
+    Col<in_T> eigval;
+    Mat<in_T> eigvec;
+    
+    const bool eig_status = eig_sym_helper(eigval, eigvec, A, 'd', "logmat()");
+    
+    if(eig_status)
+      {
+      // ensure each eigenvalue is > 0
+      
+      const uword N          = eigval.n_elem;
+      const in_T* eigval_mem = eigval.memptr();
+      
+      bool all_pos = true;
+      
+      for(uword i=0; i<N; ++i)  { all_pos = (eigval_mem[i] <= in_T(0)) ? false : all_pos; }
+      
+      if(all_pos)
+        {
+        eigval = log(eigval);
+        
+        out = conv_to< Mat<out_T> >::from( eigvec * diagmat(eigval) * eigvec.t() );
+        
+        return true;
+        }
+      }
+    
+    arma_extra_debug_print("warning: sympd optimisation failed");
+    
+    // fallthrough if eigen decomposition failed or an eigenvalue is zero
+    }
   
-  const uword n_elem = P.get_n_elem();
+  
+  Mat<out_T> S(A.n_rows, A.n_cols);
+  
+  const  in_T* Amem = A.memptr();
+        out_T* Smem = S.memptr();
+  
+  const uword n_elem = A.n_elem;
   
   for(uword i=0; i<n_elem; ++i)
     {
-    Smem[i] = std::complex<in_T>( Pea[i] );
+    Smem[i] = std::complex<in_T>( Amem[i] );
     }
   
   return op_logmat_cx::apply_common(out, S, n_iters);
@@ -136,7 +200,7 @@ op_logmat_cx::apply(Mat<typename T1::elem_type>& out, const Op<T1,op_logmat_cx>&
   
   if(status == false)
     {
-    out.reset();
+    out.soft_reset();
     arma_stop_runtime_error("logmat(): transformation failed");
     }
   }
@@ -204,6 +268,7 @@ op_logmat_cx::apply_direct(Mat<typename T1::elem_type>& out, const Base<typename
   {
   arma_extra_debug_sigprint();
   
+  typedef typename T1::pod_type   T;
   typedef typename T1::elem_type eT;
   
   Mat<eT> S = expr.get_ref();
@@ -223,6 +288,58 @@ op_logmat_cx::apply_direct(Mat<typename T1::elem_type>& out, const Base<typename
     return true;
     }
   
+  if(S.is_diagmat())
+    {
+    const uword N = S.n_rows;
+    
+    out.zeros(N,N);  // aliasing can't happen as S is generated
+    
+    for(uword i=0; i<N; ++i)  { out.at(i,i) = std::log( S.at(i,i) ); }
+    
+    return true;
+    }
+  
+  #if defined(ARMA_OPTIMISE_SYMPD)
+    const bool try_sympd = sympd_helper::guess_sympd_anysize(S);
+  #else
+    const bool try_sympd = false;
+  #endif
+  
+  if(try_sympd)
+    {
+    // if matrix S is sympd, all its eigenvalues are positive
+    
+    Col< T> eigval;
+    Mat<eT> eigvec;
+    
+    const bool eig_status = eig_sym_helper(eigval, eigvec, S, 'd', "logmat()");
+    
+    if(eig_status)
+      {
+      // ensure each eigenvalue is > 0
+      
+      const uword N          = eigval.n_elem;
+      const T*    eigval_mem = eigval.memptr();
+      
+      bool all_pos = true;
+      
+      for(uword i=0; i<N; ++i)  { all_pos = (eigval_mem[i] <= T(0)) ? false : all_pos; }
+      
+      if(all_pos)
+        {
+        eigval = log(eigval);
+        
+        out = eigvec * diagmat(eigval) * eigvec.t();
+        
+        return true;
+        }
+      }
+    
+    arma_extra_debug_print("warning: sympd optimisation failed");
+    
+    // fallthrough if eigen decomposition failed or an eigenvalue is zero
+    }
+  
   return op_logmat_cx::apply_common(out, S, n_iters);
   }
 
@@ -320,7 +437,7 @@ op_logmat_cx::helper(Mat<eT>& A, const uword m)
   vec eigval;
   mat eigvec;
   
-  const bool eig_ok = eig_sym(eigval, eigvec, tmp);
+  const bool eig_ok = eig_sym_helper(eigval, eigvec, tmp, 'd', "logmat()");
   
   if(eig_ok == false)  { arma_extra_debug_print("logmat(): eig_sym() failed"); return false; }
   
@@ -338,7 +455,7 @@ op_logmat_cx::helper(Mat<eT>& A, const uword m)
     // B += weights(i) * solve( (nodes(i)*A + eye< Mat<eT> >(N,N)), A );
     
     //const bool solve_ok = solve( X, (nodes(i)*A + eye< Mat<eT> >(N,N)), A, solve_opts::fast );
-    const bool solve_ok = solve( X, trimatu(nodes(i)*A + eye< Mat<eT> >(N,N)), A );
+    const bool solve_ok = solve( X, trimatu(nodes(i)*A + eye< Mat<eT> >(N,N)), A, solve_opts::no_approx );
     
     if(solve_ok == false)  { arma_extra_debug_print("logmat(): solve() failed"); return false; }
     
@@ -363,7 +480,7 @@ op_logmat_sympd::apply(Mat<typename T1::elem_type>& out, const Op<T1,op_logmat_s
   
   if(status == false)
     {
-    out.reset();
+    out.soft_reset();
     arma_stop_runtime_error("logmat_sympd(): transformation failed");
     }
   }
@@ -390,7 +507,7 @@ op_logmat_sympd::apply_direct(Mat<typename T1::elem_type>& out, const Base<typen
     Col< T> eigval;
     Mat<eT> eigvec;
     
-    const bool status = auxlib::eig_sym_dc(eigval, eigvec, X);
+    const bool status = eig_sym_helper(eigval, eigvec, X, 'd', "logmat_sympd()");
     
     if(status == false)  { return false; }
     
@@ -399,7 +516,7 @@ op_logmat_sympd::apply_direct(Mat<typename T1::elem_type>& out, const Base<typen
     
     bool all_pos = true;
     
-    for(uword i=0; i<N; ++i)  { all_pos = (eigval_mem[i] < T(0)) ? false : all_pos; }
+    for(uword i=0; i<N; ++i)  { all_pos = (eigval_mem[i] <= T(0)) ? false : all_pos; }
     
     if(all_pos == false)  { return false; }
     
